using System;
using System.Globalization;
using System.Xml;
using System.Web.Services.Protocols;

using Microsoft.Web.Services3;
using Microsoft.Web.Services3.Xml;

namespace WSCompression
{
    internal sealed class Compression : SoapHeader
    {
        private CompressionMode _compressionMode;
        private CompressionLevel _compressionLevel;
        private CompressionMethod _compressionMethod;
        private ReferenceList _referenceList;

        public Compression( CompressionMode compressionMode, CompressionLevel compressionLevel )
        {
            _compressionMode = compressionMode;
            _compressionLevel = compressionLevel;
            _compressionMethod = new CompressionMethod( compressionMode, compressionLevel );
            
            MustUnderstand = true;
        }

        public Compression( XmlElement element )
        {
            _compressionMode = CompressionMode.None;
            _compressionLevel = CompressionLevel.NoCompression;
            LoadXml( element );
        }

        public CompressionMethod Method
        {
            get
            {
                return _compressionMethod;
            }
        }
        
        public CompressionMode Mode
        {
            get
            {
                return _compressionMode;
            }
        }

        public CompressionLevel Level
        {
            get
            {
                return _compressionLevel;
            }
        }

        public ReferenceList ReferenceList
        {
            get
            {
                if ( _referenceList == null )
                    _referenceList = new ReferenceList( );
                return _referenceList;
            }
        }

        public static bool IsValidElement( XmlElement element )
        {
            return ( element.LocalName == WSCompression.ElementNames.Compression && element.NamespaceURI == WSCompression.NamespaceURI );
        }

        public XmlElement GetXml( XmlDocument document )
        {
            Soap12 soap12Attributes = new Soap12( );

            XmlElement compressionElement = document.CreateElement( WSCompression.Prefix, WSCompression.ElementNames.Compression, WSCompression.NamespaceURI );
            if ( !String.IsNullOrEmpty( Actor ) )
                soap12Attributes.SetActorAttribute( document, compressionElement, Actor );

            if ( MustUnderstand )
                soap12Attributes.SetMustUnderstand( document, compressionElement, true );

            if ( Method != null )
                compressionElement.AppendChild( Method.GetXml( document ) );

            if ( _referenceList != null )
                compressionElement.AppendChild( _referenceList.GetXml( document ) );

            return compressionElement;
        }

        public void LoadXml( XmlElement element )
        {
            if ( element == null )
                throw new ArgumentNullException( "element" );

            if ( !Compression.IsValidElement( element ) )
                throw new ArgumentException( "Invalid compression header", "element" );

            Soap12 soapAttributes = new Soap12( );
            
            string actor = soapAttributes.GetActorAttribute( element );
            if ( String.IsNullOrEmpty( actor ) )
                Actor = actor;

            MustUnderstand = soapAttributes.GetMustUnderstand( element );

            _compressionMethod = null;
            _referenceList = null;

            foreach ( XmlNode childNode in element.ChildNodes )
            {
                XmlElement childElement = childNode as XmlElement;
                if ( childElement != null )
                {
                    if ( ReferenceList.IsValidElement( childElement ) )
                    {
                        _referenceList = new ReferenceList( childElement );
                    }
                    else if ( CompressionMethod.IsValidElement( childElement ) )
                    {
                        _compressionMethod = new CompressionMethod( childElement );
                    }
                    else
                    {
                        throw new CompressionFault( CompressionFault.InvalidCompressionMessage, CompressionFault.InvalidCompressionCode );
                    }
                }
            }

            if ( _referenceList == null || _compressionMethod == null )
                throw new CompressionFault( CompressionFault.InvalidCompressionMessage, CompressionFault.InvalidCompressionCode );

            try
            {
                _compressionMode = CompressionMethod.GetCompressionMode( Method.Algorithm );
                _compressionLevel = CompressionMethod.GetCompressionLevel( Method.Level );
            }
            catch ( ArgumentOutOfRangeException )
            {
                throw new CompressionFault( CompressionFault.UnsupportedAlgorithmMessage, CompressionFault.UnsupportedAlgorithmCode );
            }

            if ( ReferenceList.Count != 1 )
                throw new CompressionFault( CompressionFault.InvalidCompressionMessage, CompressionFault.InvalidCompressionCode );
        }

        public void Compress( SoapEnvelope document )
        {
            if ( document == null )
                throw new ArgumentNullException( "document" );

            string compressedBodyURI = ReferenceList.GenerateNewReference( );
            ReferenceList.AddReference( compressedBodyURI );
            XmlElement compressionElement = GetXml( document );
            XmlElement body = document.Body;

            CompressedData compressedData = new CompressedData( Mode, Level, compressedBodyURI );
            XmlElement compressedDataElement = compressedData.GetCompressedXml( document, body );

            XmlNode currentNode = body.FirstChild;
            while ( ( currentNode != null ) )
            {
                XmlNode nextNode = currentNode.NextSibling;
                if ( currentNode is XmlElement )
                    body.RemoveChild( currentNode );
                currentNode = nextNode;
            }

            body.AppendChild( compressedDataElement );

            document.CreateHeader( ).AppendChild( compressionElement );
        }

        public static void Decompress( XmlElement compressionElement )
        {
            SoapEnvelope envelope = compressionElement.OwnerDocument as SoapEnvelope;
            if ( envelope == null )
                throw new ArgumentException( "Requires a SoapEnvelope document", "compressionElement" );

            Compression compression = new Compression( compressionElement );

            string reference = compression.ReferenceList[ 0 ];

            XmlElement referenceData = null;
            for ( int i = 0; i < envelope.Body.ChildNodes.Count; ++i )
            {
                XmlElement element = envelope.Body.ChildNodes[ i ] as XmlElement;
                if ( element != null && CompressedData.IsValidElement( element ) )
                {
                    if ( referenceData != null )
                        throw new CompressionFault( CompressionFault.InvalidCompressionMessage, CompressionFault.InvalidCompressionCode );

                    string id = element.GetAttribute( WSCompression.AttributeNames.Id );
                    if ( id != "#" + reference )
                        throw new CompressionFault( CompressionFault.ReferencedDataNotFoundMessage, CompressionFault.ReferencedDataNotFoundCode );

                    referenceData = element;
                }
            }

            if ( referenceData == null )
                throw new CompressionFault( CompressionFault.ReferencedDataNotFoundMessage, CompressionFault.ReferencedDataNotFoundCode );

            CompressedData compressedData = new CompressedData( compression.Mode, compression.Level, referenceData );
            referenceData.ParentNode.InnerXml = compressedData.Decompress( referenceData );

            envelope.Header.RemoveChild( compressionElement );
        }
    }
}