using System;
using System.Xml;

using Microsoft.Web.Services3.Xml;

namespace WSCompression
{
    internal class CompressionMethod : IXmlElement
    {
        private string _algorithm;
        private string _level;

        public CompressionMethod( CompressionMode compressionMode, CompressionLevel compressionLevel )
        {
            _algorithm = GetAlgorithm( compressionMode );
            _level = GetLevel( compressionLevel );
        }

        public CompressionMethod( XmlElement element )
        {
            LoadXml( element );
        }

        public static bool IsValidElement( XmlElement element )
        {
            return ( element.LocalName == WSCompression.ElementNames.CompressionMethod && element.NamespaceURI == WSCompression.NamespaceURI );
        }

        public string Algorithm { get { return _algorithm; } }

        public static string GetAlgorithm( CompressionMode compressionMode )
        {
            switch ( compressionMode )
            {
                case CompressionMode.GZip:
                    return WSCompression.Algorithm.GZip;
                case CompressionMode.Deflate:
                    return WSCompression.Algorithm.Deflate;
                default:
                    throw new ArgumentOutOfRangeException( "compressionMode" );
            }
        }

        public static CompressionMode GetCompressionMode( string algorithm )
        {
            switch ( algorithm )
            {
                case WSCompression.Algorithm.GZip:
                    return CompressionMode.GZip;
                case WSCompression.Algorithm.Deflate:
                    return CompressionMode.Deflate;
                default:
                    throw new ArgumentOutOfRangeException( "compressionMode" );
            }
        }

        public string Level
        {
            get
            { 
                return _level;
            }
        }

        public static string GetLevel( CompressionLevel compressionLevel )
        {
            switch ( compressionLevel )
            {
                case CompressionLevel.Fast:
                    return WSCompression.Level.Fast;
                case CompressionLevel.Maximum:
                    return WSCompression.Level.Maximum;
                case CompressionLevel.NoCompression:
                    return WSCompression.Level.NoCompression;
                case CompressionLevel.Normal:
                    return WSCompression.Level.Normal;
                default:
                    throw new ArgumentOutOfRangeException( "compressionLevel" );
            }
        }

        public static CompressionLevel GetCompressionLevel( string level )
        {
            switch ( level )
            {
                case WSCompression.Level.Fast:
                    return CompressionLevel.Fast;
                case WSCompression.Level.Maximum:
                    return CompressionLevel.Maximum;
                case WSCompression.Level.NoCompression:
                    return CompressionLevel.NoCompression;
                case WSCompression.Level.Normal:
                    return CompressionLevel.Normal;
                default:
                    throw new ArgumentOutOfRangeException( "level" );
            }
        }

        #region IXmlElement Members

        public void LoadXml( XmlElement element )
        {
            if ( element == null )
                throw new ArgumentNullException( "element" );

            if ( !IsValidElement( element ) )
                throw new ArgumentException( "element" );

            _algorithm = element.GetAttribute( WSCompression.AttributeNames.Algorithm );
            _level = element.GetAttribute( WSCompression.AttributeNames.Level );
        }

        public XmlElement GetXml( XmlDocument document )
        {
            if ( document == null )
                throw new ArgumentNullException( "document" );

            XmlElement compressionMethodElement = document.CreateElement( WSCompression.Prefix, WSCompression.ElementNames.CompressionMethod, WSCompression.NamespaceURI );
            compressionMethodElement.SetAttribute( WSCompression.AttributeNames.Algorithm, Algorithm );
            compressionMethodElement.SetAttribute( WSCompression.AttributeNames.Level, Level );
            return compressionMethodElement;
        }

        #endregion
    }
}